;=====================================================
;TRESSTRDBG (WINDOWED) MAIN REAL-TIME DISPLAY CODE
;=====================================================
;[ebp-4] = DC for current screen writes
;[ebp-8] = DDRAW SURFACE OBJECT (0 if GUI)
;[ebp-12] = Window handle
;[ebp-16] = possible dump file handle
;[ebp-20] = temporary var
;[ebp-24] = start of parameters address
;[ebp-28] = string address
;[ebp-32] = converted string length

TRES_STR_DBG_MAIN:
push ebp
mov ebp,esp
sub esp,150
push edi
push esi
push ebx
mov esi,[BaseData]

invoke FindWindow,WindowClassName,WindowName
mov [ebp-12],eax		;if window doesn't exist, string is
				;displayed on desktop
invoke SetForegroundWindow,eax	;sets focus

;=================================================
;------- test parameter validity
;=================================================
lea eax,[ebp+12]		;2nd param
mov edx,[ebp+8] 		;1rst param (to test)
    cmp edx,[DATASEC_STARTADDR]
    jb Param1NotInDataSec_SDBG
    cmp edx,[DATASEC_ENDADDR]
    jb ParamValidated_SDBG

Param1NotInDataSec_SDBG:
lea eax,[ebp+16]		;3rd param
mov edx,[ebp+12]		;2nd param (to test)
    cmp edx,[DATASEC_STARTADDR]
    jb Exit_SDBG
    cmp edx,[DATASEC_ENDADDR]
    jnb Exit_SDBG

ParamValidated_SDBG :
mov [ebp-24],eax
mov [ebp-28],edx

or ecx,0FFFFFFFFh
CountLoop2:		;get string length in ecx (before sprintf)
inc ecx
cmp BYTE [edx+ecx],0
jnz CountLoop2

cmp ecx,0
jz Exit_SDBG
cmp ecx,80		   ;accept string length of 80 bytes max
ja Exit_SDBG		   ;its size can grow with sprintf = use caution

;=================================================
;------- get device context
;=================================================
invoke GetDC,[ebp-12]
test eax,eax
mov [ebp-4],eax
jz Exit_SDBG

invoke SetTextColor,eax,0
;=================================================
;-------- main TRESSTRDBG code (print to screen)
;=================================================
PrintText_SDBG:
sub esp,48		;max 48 bytes of parameters (assumed)

push edi
push esi
mov esi,[ebp-24]	;start of parameters address
lea edi,[esp+8] 	;to local space
mov ecx,12
rep movsd	;copy parameters passed to this function even if none
pop esi 	;(offsets the cycles that would be used to check for parameters)

mov ecx,25
lea edi,[ebp-150+16]	;put space chars to main string buffer
mov eax,20202020h	;->needed to make sure strings are clear on-screen
rep stosd
pop edi

lea edx,[ebp-150+16]		     ;destination for converted string (right after ADDRESS)
invoke sprintf,edx,DWORD [ebp-28]    ;convert string w/parameters
add esp,(48+8)			     ;sprintf does not pop parameters

lea edx,[ebp-150+16]
or ecx,0FFFFFFFFh
   CountLoop1_SDBG:	   ;counts final string length and checks for bad
   inc ecx		   ;chars (and decides finally if string is valid)
   mov al,[edx+ecx]
   cmp al,0Ah		   ;annoying char that pops up in many strings
   jnz NotCharX_1_SDBG
   mov BYTE [edx+ecx],20h
   jmp CountLoop1_SDBG
NotCharX_1_SDBG:
test al,al
jnz CountLoop1_SDBG
mov [ebp-32],ecx	;save string length (converted string returned from sprintf)
mov BYTE [ecx+edx],20h	;put space over null-terminator

invoke TextOut,DWORD [ebp-4],0,0,edx,100	;print main string w/100-byte length

mov DWORD [ebp-150],078300A0Dh	    ;carriage return + '0x'
mov DWORD [ebp-150+4], '0000'
mov DWORD [ebp-150+8], '0000'
mov DWORD [ebp-150+12], 0A0D2020h   ;2 spaces+another return

lea edx,[ebp-150+11]		 ;last '0'
mov eax,[ebp+4] 		 ;return address (to caller)
    ConvertAddrLoop_SDBG:	 ;converts DWORD return address to string (hex)
    mov cl,al
    and cl,0Fh
    cmp cl,09h
    jna NoNeedAdd_SDBG
    add cl,7			 ;for letters
    NoNeedAdd_SDBG:
    add [edx],cl
    shr eax,4
    dec edx
    cmp BYTE [edx],'x'
    jnz ConvertAddrLoop_SDBG

lea edx,[ebp-150+2]
invoke TextOut,DWORD [ebp-4],0,10h,edx,12	;print the ADDRESS ("RET:")

;=================================================
;--------- Exit code
;=================================================
Exit_Safe_SDBG:

invoke ReleaseDC,DWORD [ebp-12],DWORD [ebp-4]

push [UseTRESSTRDBG]
call WaitForInterval_SDBG

Exit_SDBG:
pop ebx
pop esi
pop edi
mov esp,ebp
pop ebp
ret
;=================================================
;------------- Wait code
;=================================================
PAUSEKEY	=	VK_NUMPAD5
RUNTHROUGHKEY	=	VK_NUMPAD8
SAVETOTEXTKEY	=	VK_NUMPAD1
EMEREXITKEY	=	VK_NUMPAD9

WaitForInterval_SDBG:	;parameter = interval (wait time)
push esi
invoke GetTickCount
mov esi,eax

WaitLoop_SDBG:
invoke GetAsyncKeyState,EMEREXITKEY
	test eax,80000000h	;high-order bit set if key pressed
	jnz Exit_KeyIsDown_SDBG
invoke GetAsyncKeyState,RUNTHROUGHKEY
	test eax,80000000h
	jnz Run_KeyIsDown_SDBG
invoke GetAsyncKeyState,PAUSEKEY
	test eax,80000000h
	jnz Pause_KeyIsDown_SDBG

invoke GetTickCount
sub eax,esi
cmp eax,[esp+8]
jb WaitLoop_SDBG
pop esi
ret 4

Pause_KeyIsDown_SDBG:		     ;pauses while F5 is pressed
push edi
xor edi,edi
Pause_KeyIsDown_LOOP_SDBG:
	test edi,edi
	jnz NoDump1_SDBG
	invoke GetAsyncKeyState,SAVETOTEXTKEY	;is write to file key down?
	test eax,80000000h
	jz NoDump1_SDBG
	mov edi,eax
	call WRITETOFILE_SDBG
	NoDump1_SDBG:
invoke GetAsyncKeyState,PAUSEKEY
test eax,80000000h
jnz Pause_KeyIsDown_LOOP_SDBG		;wait until F5 comes back up
pop edi
pop esi
ret 4

Run_KeyIsDown_SDBG:		     ;exits loop quickly, can make dump
	invoke GetAsyncKeyState,SAVETOTEXTKEY
	test eax,80000000h
	jz NoDump2_SDBG
	call WRITETOFILE_SDBG
	NoDump2_SDBG:
pop esi
ret 4

Exit_KeyIsDown_SDBG:
pop esi
add esp,4
pop ebx
pop esi
pop edi
mov esp,ebp
pop ebp
invoke ExitProcess,0		;force exit if F7 pressed
ret

;=================================================
;------------- Write data to file
;=================================================
WRITETOFILE_SDBG:
invoke CreateFile,SDBGDUMPFILE,GENERIC_WRITE,0,0,OPEN_ALWAYS,\
			FILE_ATTRIBUTE_NORMAL,0
mov [ebp-16],eax		;file handle
invoke SetFilePointer,eax,0,0,FILE_END

mov ecx,[ebp-32]		;string length
lea edx,[ebp-150+16]		;string start
mov WORD [edx+ecx],0A0Dh	;place ending carriage return

add ecx,18		;16 for retaddr, 2 for end carriage ret
			;ecx=number of bytes to write
lea eax,[ebp-150]	;bufferstart (ret addr)
lea edx,[ebp-20]	;#bytesread (unused)

invoke WriteFile,DWORD [ebp-16],eax,ecx,edx,0
invoke CloseHandle,DWORD [ebp-16]
ret
;=================================================